 configure.ac                             |    6 +
 include/vlc_codec.h                      |    3 +
 modules/codec/aac.c                      |  551 ++++++++++++++++++++++++++++++
 modules/stream_out/transcode/transcode.c |   20 +-
 4 files changed, 578 insertions(+), 2 deletions(-)

diff --git a/configure.ac b/configure.ac
index 49e0901..1681b3c 100644
--- a/configure.ac
+++ b/configure.ac
@@ -2612,6 +2612,12 @@ dnl
 PKG_ENABLE_MODULES_VLC([TWOLAME], [], [twolame], [MPEG Audio Layer 2 encoder], [auto], [], [], [ -DLIBTWOLAME_STATIC])
 
 dnl
+dnl fdk-aac encoder plugin
+dnl
+PKG_ENABLE_MODULES_VLC([FDKAAC], [], [fdk-aac], [FDK-AAC encoder], [auto])
+VLC_ADD_LIBS([FDKAAC],[ -lfdk-aac])
+
+dnl
 dnl  QuickTime plugin
 dnl
 AC_ARG_ENABLE(quicktime,
diff --git a/include/vlc_codec.h b/include/vlc_codec.h
index beac8fd..0750292 100644
--- a/include/vlc_codec.h
+++ b/include/vlc_codec.h
@@ -173,6 +173,9 @@ struct encoder_t
 
     /* Encoder config */
     config_chain_t *p_cfg;
+    
+    /* Does the encoder need purging at EOF? */
+    bool b_purge_required;
 };
 
 /**
diff --git a/modules/codec/aac.c b/modules/codec/aac.c
new file mode 100644
index 0000000..0ca3aae
--- /dev/null
+++ b/modules/codec/aac.c
@@ -0,0 +1,551 @@
+/*****************************************************************************
+ * aac.c: FDK-AAC Encoder plugin for vlc.
+ *****************************************************************************
+ * Copyright (C) 2012 the SipRadius team
+ *
+ * Authors: Sergio Ammirata <sergio@ammirata.net>
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+ * 
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ * 
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ *****************************************************************************/
+
+/*****************************************************************************
+ * Preamble
+ *****************************************************************************/
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
+
+#include <stdio.h>
+#include <stdint.h>
+#include <unistd.h>
+#include <stdlib.h>
+#include <fdk-aac/aacenc_lib.h>
+
+#include <vlc_common.h>
+#include <vlc_plugin.h>
+#include <vlc_codec.h>
+
+static int OpenEncoder( vlc_object_t * );
+static void CloseEncoder( vlc_object_t * );
+
+#define ENC_CFG_PREFIX "sout-fdkaac-"
+
+#define AOT_TEXT N_("Encoder Profile")
+#define AOT_LONGTEXT N_( "Encoder Algorithm to use" )
+
+#define SIDEBAND_TEXT N_("Enable spectral band replication")
+#define SIDEBAND_LONGTEXT N_( "This is an optional feature only for the AAC-ELD profile" )
+
+#define VBR_QUALITY_TEXT N_("VBR Quality")
+#define VBR_QUALITY_LONGTEXT N_( "Quality of the VBR Encoding (0=cbr, 1-5 constant quality vbr, 5 is best" )
+
+#define AFTERBURNER_TEXT N_("Enable afterburner library")
+#define AFTERBURNER_LONGTEXT N_( "This library will produce higher quality audio at the expense of additional CPU usage (default is enabled)" )
+
+#define BITRATE_TEXT N_("CBR Bitrate")
+#define BITRATE_LONGTEXT N_( "Bitrate of desired stream (in bps)" )
+
+#define SIGNALING_TEXT N_("Signaling mode of the extension AOT")
+#define SIGNALING_LONGTEXT N_( "1 is explicit for SBR and implicit for PS (default), 2 is explicit hierarchical" )
+
+#define  CH_ORDER_MPEG 0  /*!< MPEG channel ordering (e. g. 5.1: C, L, R, SL, SR, LFE)           */
+#define  CH_ORDER_WAV 1   /*!< WAV fileformat channel ordering (e. g. 5.1: L, R, C, LFE, SL, SR) */
+#define  CH_ORDER_WG4 2   /*!< WG4 fileformat channel ordering (e. g. 5.1: C, L, R, SL, SR, LFE) */
+
+#define PROFILE_AAC_LC 2
+#define PROFILE_AAC_HE 5
+#define PROFILE_AAC_HE_v2 29
+#define PROFILE_AAC_LD 23
+#define PROFILE_AAC_ELD 39
+
+#define SIGNALING_COMPATIBLE 1
+#define SIGNALING_HIERARCHICAL 2
+
+/* Wg4
+AOUT_CHAN_LEFT, 
+AOUT_CHAN_RIGHT,
+AOUT_CHAN_MIDDLELEFT, 
+AOUT_CHAN_MIDDLERIGHT,
+AOUT_CHAN_REARLEFT, 
+AOUT_CHAN_REARRIGHT, 
+AOUT_CHAN_REARCENTER,
+AOUT_CHAN_CENTER, 
+AOUT_CHAN_LFE, 
+*/
+
+static const int pi_aot_values[] = { PROFILE_AAC_LC, PROFILE_AAC_HE, PROFILE_AAC_HE_v2, PROFILE_AAC_LD, PROFILE_AAC_ELD };
+static const char *const ppsz_aot_descriptions[] =
+{ N_("AAC-LC"), N_("HE-AAC"), N_("HE-AAC-v2"), N_("AAC-LD"), N_("AAC-ELD") };
+//HE-AAC v2 == HE-AAC + PS == AAC-LC + SBR + PS
+//HE-AAC == AAC-LC + SBR
+
+vlc_module_begin ()
+    set_shortname( N_("FDKAAC") )
+    set_description( N_("FDK-AAC Audio encoder") )
+    set_capability( "encoder", 50 )
+    set_callbacks( OpenEncoder, CloseEncoder )
+    add_shortcut( "fdkaac" )
+    set_category( CAT_INPUT )
+    set_subcategory( SUBCAT_INPUT_ACODEC )
+    add_integer( ENC_CFG_PREFIX "profile", PROFILE_AAC_LC, AOT_TEXT,
+             AOT_LONGTEXT, false )
+    change_integer_list( pi_aot_values, ppsz_aot_descriptions );
+    add_bool( ENC_CFG_PREFIX "sbr", false, SIDEBAND_TEXT,
+              SIDEBAND_LONGTEXT, false )
+    add_integer( ENC_CFG_PREFIX "vbr", 0, VBR_QUALITY_TEXT, 
+              VBR_QUALITY_LONGTEXT, false )
+    change_integer_range (0, 5)
+    change_safe ()
+    add_bool( ENC_CFG_PREFIX "afterburner", true, AFTERBURNER_TEXT,
+              AFTERBURNER_LONGTEXT, false )
+    add_integer( ENC_CFG_PREFIX "signaling", SIGNALING_COMPATIBLE, SIGNALING_TEXT,
+             SIGNALING_LONGTEXT, false )
+    change_integer_range (0, 2)
+    change_safe ()
+vlc_module_end ()
+
+/*****************************************************************************
+ * Local prototypes
+ *****************************************************************************/
+static block_t *EncodeAudio( encoder_t *p_enc, aout_buffer_t *p_buf );
+static void ConfigureChannelOrder(int *, int, uint32_t );
+
+static const char *const ppsz_enc_options[] = {
+    "profile", "sbr", "vbr", "afterburner", "signaling", NULL
+};
+
+/*****************************************************************************
+ * encoder_sys_t : aac encoder descriptor
+ *****************************************************************************/
+struct encoder_sys_t
+{
+    HANDLE_AACENCODER handle;
+    int i_aot;
+    bool b_eld_sbr;
+    int i_vbr;
+    bool b_afterburner;
+    int i_signaling;
+    int i_encoderdelay;
+    int i_frame_size;
+    double d_compression_ratio;
+    mtime_t i_pts_last;
+};
+
+static const char *aac_get_errorstring(AACENC_ERROR erraac)
+{
+    switch (erraac) {
+    case AACENC_OK:
+        return "No error";
+    case AACENC_INVALID_HANDLE:
+        return "Invalid handle";
+    case AACENC_MEMORY_ERROR:
+        return "Memory allocation error";
+    case AACENC_UNSUPPORTED_PARAMETER:
+        return "Unsupported parameter";
+    case AACENC_INVALID_CONFIG:
+        return "Invalid config";
+    case AACENC_INIT_ERROR:
+        return "Initialization error";
+    case AACENC_INIT_AAC_ERROR:
+        return "AAC library initialization error";
+    case AACENC_INIT_SBR_ERROR:
+        return "SBR library initialization error";
+    case AACENC_INIT_TP_ERROR:
+        return "Transport library initialization error";
+    case AACENC_INIT_META_ERROR:
+        return "Metadata library initialization error";
+    case AACENC_ENCODE_ERROR:
+        return "Encoding error";
+    case AACENC_ENCODE_EOF:
+        return "End of file";
+    default:
+        return "Unknown error";
+    }
+}
+
+/*****************************************************************************
+ * OpenDecoder: open the encoder.
+ *****************************************************************************/
+static int OpenEncoder( vlc_object_t *p_this )
+{
+    encoder_t *p_enc = (encoder_t *)p_this;
+    encoder_sys_t *p_sys;
+    CHANNEL_MODE mode;
+    AACENC_ERROR erraac;
+    bool b_profile_selected = false;
+    int sce = 0, cpe = 0;
+    int i_profile, i_bitrate;
+    
+    if( p_enc->fmt_out.i_codec != VLC_FOURCC( 'l', 'a', 'a', 'c' ) && 
+        p_enc->fmt_out.i_codec != VLC_FOURCC( 'h', 'a', 'a', 'c' ) && 
+        p_enc->fmt_out.i_codec != VLC_FOURCC( 's', 'a', 'a', 'c' ) &&
+        p_enc->fmt_out.i_codec != VLC_CODEC_MP4A )
+    {
+        return VLC_EGENERIC;
+    }
+    else if ( p_enc->fmt_out.i_codec == VLC_FOURCC( 'l', 'a', 'a', 'c' ) )
+    {
+        b_profile_selected = true;
+        i_profile = PROFILE_AAC_LC;
+    }
+    else if ( p_enc->fmt_out.i_codec == VLC_FOURCC( 'h', 'a', 'a', 'c' ) )
+    {
+        b_profile_selected = true;
+        i_profile = PROFILE_AAC_HE;
+    }
+    else if ( p_enc->fmt_out.i_codec == VLC_FOURCC( 's', 'a', 'a', 'c' ) )
+    {
+        b_profile_selected = true;
+        i_profile = PROFILE_AAC_HE_v2;
+    }   
+
+    msg_Info(p_enc, "Initializing AAC Encoder, %i channels", p_enc->fmt_in.audio.i_channels);
+
+    switch (p_enc->fmt_in.audio.i_channels) {
+    case 1: mode = MODE_1;       sce = 1; cpe = 0; break;
+    case 2: mode = MODE_2;       sce = 0; cpe = 1; break;
+    case 3: mode = MODE_1_2;     sce = 1; cpe = 1; break;
+    case 4: mode = MODE_1_2_1;   sce = 2; cpe = 1; break;
+    case 5: mode = MODE_1_2_2;   sce = 1; cpe = 2; break;
+    case 6: mode = MODE_1_2_2_1; sce = 2; cpe = 2; break;
+    case 8: mode = MODE_1_2_2_2_1; sce = 2; cpe = 3; break;
+    default:
+        msg_Err( p_enc, "we do not support > 8 input channels, this input has %i", 
+                        p_enc->fmt_in.audio.i_channels );
+        return VLC_EGENERIC;
+    }
+	
+    /* Allocate the memory needed to store the encoder's structure */
+    if( ( p_sys = (encoder_sys_t *)malloc(sizeof(encoder_sys_t)) ) == NULL )
+        return VLC_ENOMEM;
+    p_enc->p_sys = p_sys;
+    
+    p_enc->pf_encode_audio = EncodeAudio;
+
+    p_enc->fmt_in.i_codec = VLC_CODEC_S16N;
+    p_enc->fmt_out.i_cat = AUDIO_ES;
+    p_enc->fmt_out.i_codec = VLC_CODEC_MP4A;
+
+    config_ChainParse( p_enc, ENC_CFG_PREFIX, ppsz_enc_options, p_enc->p_cfg );
+ 
+    if ( b_profile_selected == false )
+        p_sys->i_aot = var_CreateGetInteger( p_enc, ENC_CFG_PREFIX "profile" );
+    else
+        p_sys->i_aot = i_profile;
+    p_sys->b_eld_sbr = var_CreateGetBool( p_enc, ENC_CFG_PREFIX "sbr" );
+    p_sys->i_vbr = var_CreateGetInteger( p_enc, ENC_CFG_PREFIX "vbr" );
+    p_sys->b_afterburner = var_CreateGetBool( p_enc, ENC_CFG_PREFIX "afterburner" );
+    p_sys->i_signaling = var_CreateGetInteger( p_enc, ENC_CFG_PREFIX "signaling" );
+    p_sys->i_pts_last = 0;
+ 
+    if ((p_sys->i_aot == PROFILE_AAC_HE || p_sys->i_aot == PROFILE_AAC_HE_v2) && p_sys->i_vbr > 3)
+    {
+        msg_Warn(p_enc, "Maximum VBR quality for this profile is 3, setting vbr=3");
+        p_sys->i_vbr = 3;
+    }
+    if ((erraac = aacEncOpen(&p_sys->handle, 0, p_enc->fmt_in.audio.i_channels)) != AACENC_OK) {
+        msg_Err(p_enc, "Unable to open encoder: %s", aac_get_errorstring(erraac));
+        free( p_sys );
+        return VLC_EGENERIC;
+    }
+    if ( p_sys->i_aot == PROFILE_AAC_HE_v2 && p_enc->fmt_in.audio.i_channels != 2 )
+    {
+        msg_Err(p_enc, "The HE-AAC-v2 profile can only be used with stereo sources");
+        free( p_sys );
+        return VLC_EGENERIC;
+    }
+    if ( p_sys->i_aot == PROFILE_AAC_ELD && p_enc->fmt_in.audio.i_channels != 2 )
+    {
+        msg_Err(p_enc, "The ELD-AAC profile can only be used with stereo sources");
+        free( p_sys );
+        return VLC_EGENERIC;
+    }
+    if ((erraac = aacEncoder_SetParam(p_sys->handle, AACENC_AOT, p_sys->i_aot)) != AACENC_OK) {
+        msg_Err(p_enc, "Unable to set the Profile %i: %s", p_sys->i_aot, aac_get_errorstring(erraac));
+        free( p_sys );
+        return VLC_EGENERIC;
+    }
+    if (p_sys->i_aot == PROFILE_AAC_ELD && p_sys->b_eld_sbr) {
+        if ((erraac = aacEncoder_SetParam(p_sys->handle, AACENC_SBR_MODE, 1)) != AACENC_OK) {
+            msg_Err(p_enc, "Unable to set SBR mode for ELD: %s", aac_get_errorstring(erraac));
+            free( p_sys );
+            return VLC_EGENERIC;
+        }
+    }
+    if ((erraac = aacEncoder_SetParam(p_sys->handle, AACENC_SAMPLERATE, 
+    	              p_enc->fmt_out.audio.i_rate)) != AACENC_OK) {
+        msg_Err(p_enc, "Unable to set the sample rate %i: %s",p_enc->fmt_out.audio.i_rate,
+                        aac_get_errorstring(erraac));
+        free( p_sys );
+        return VLC_EGENERIC;
+    }
+    if ((erraac = aacEncoder_SetParam(p_sys->handle, AACENC_CHANNELMODE, mode)) != AACENC_OK) {
+        msg_Err(p_enc, "Unable to set the channel mode: %s", aac_get_errorstring(erraac));
+        free( p_sys );
+        return VLC_EGENERIC;
+    }
+    if ((erraac = aacEncoder_SetParam(p_sys->handle, AACENC_CHANNELORDER, CH_ORDER_WG4)) != AACENC_OK) {
+        msg_Err(p_enc, "Unable to set the sound channel order: %s", aac_get_errorstring(erraac));
+        free( p_sys );
+        return VLC_EGENERIC;
+    }
+    if (p_sys->i_vbr != 0) {
+        if ((erraac = aacEncoder_SetParam(p_sys->handle, 
+        	               AACENC_BITRATEMODE, p_sys->i_vbr)) != AACENC_OK) {
+            msg_Err(p_enc, "Unable to set the VBR bitrate mode: %s", aac_get_errorstring(erraac));
+            free( p_sys );
+            return VLC_EGENERIC;
+        }
+    } else {
+        if (p_enc->fmt_out.i_bitrate == 0) {
+            if (p_sys->i_aot == PROFILE_AAC_HE_v2) {
+                sce = 1;
+                cpe = 0;
+            }
+            i_bitrate = (96*sce + 128*cpe) * p_enc->fmt_out.audio.i_rate / 44;
+            if (p_sys->i_aot == PROFILE_AAC_HE ||
+                p_sys->i_aot == PROFILE_AAC_HE_v2 ||
+                p_sys->b_eld_sbr)
+                i_bitrate /= 2;
+            p_enc->fmt_out.i_bitrate = i_bitrate;
+            msg_Info(p_enc, "Setting optimal bitrate of %i", i_bitrate);
+        }
+        else
+        {
+            i_bitrate = p_enc->fmt_out.i_bitrate;
+        }
+        if ((erraac = aacEncoder_SetParam(p_sys->handle, AACENC_BITRATE, 
+        	                               i_bitrate)) != AACENC_OK) {
+            msg_Err(p_enc, "Unable to set the bitrate %i: %s", i_bitrate,
+                    aac_get_errorstring(erraac));
+            free( p_sys );
+            return VLC_EGENERIC;
+        }
+    }
+    if ((erraac = aacEncoder_SetParam(p_sys->handle, AACENC_TRANSMUX, 0)) != AACENC_OK) {
+        msg_Err(p_enc, "Unable to set the ADTS transmux: %s", aac_get_errorstring(erraac));
+        free( p_sys );
+        return VLC_EGENERIC;
+    }    
+    if ((erraac = aacEncoder_SetParam(p_sys->handle, AACENC_SIGNALING_MODE, 
+    	                         (int)p_sys->i_signaling)) != AACENC_OK) {
+    	/* use explicit backward compatible =1 */
+    	/* use explicit hierarchical signaling =2 */
+        msg_Err(p_enc, "Unable to set signaling mode: %s", aac_get_errorstring(erraac));
+        free( p_sys );
+        return VLC_EGENERIC;
+    }
+    if ((erraac = aacEncoder_SetParam(p_sys->handle, AACENC_AFTERBURNER, 
+    	                         (int)p_sys->b_afterburner)) != AACENC_OK) {
+        msg_Err(p_enc, "Unable to set the afterburner mode: %s", aac_get_errorstring(erraac));
+        free( p_sys );
+        return VLC_EGENERIC;
+    }
+    if ((erraac = aacEncEncode(p_sys->handle, NULL, NULL, NULL, NULL)) != AACENC_OK) {
+        msg_Err(p_enc, "Unable to initialize the encoder: %s", aac_get_errorstring(erraac));
+        free( p_sys );
+        return VLC_EGENERIC;
+    }
+    AACENC_InfoStruct info = { 0 };
+    if ((erraac = aacEncInfo(p_sys->handle, &info)) != AACENC_OK) {
+        msg_Err(p_enc, "Unable to get the encoder info: %s", aac_get_errorstring(erraac));
+        free( p_sys );
+        return VLC_EGENERIC;
+    }
+
+    p_enc->fmt_in.audio.i_bitspersample = 16;
+    p_sys->i_frame_size = info.frameLength;
+    p_sys->i_encoderdelay = info.encoderDelay; 
+
+    p_enc->fmt_out.i_extra = info.confSize;
+    if( p_enc->fmt_out.i_extra )
+    {
+        p_enc->fmt_out.p_extra = malloc( p_enc->fmt_out.i_extra );
+        if ( p_enc->fmt_out.p_extra == NULL )
+        {
+            msg_Err(p_enc, "Unable to allocate fmt_out.p_extra");
+            free( p_sys );
+            return VLC_EGENERIC;
+        }
+        memcpy( p_enc->fmt_out.p_extra, info.confBuf,
+                p_enc->fmt_out.i_extra );
+    }
+    
+    p_enc->b_purge_required = true;
+    
+    // TODO: Add more debug info to this config printout
+    msg_Dbg(p_enc, "fmt_out.p_extra = %i", p_enc->fmt_out.i_extra);
+
+    return VLC_SUCCESS;
+}
+
+/****************************************************************************
+ * EncodeAudio: the whole thing
+ ****************************************************************************/
+static block_t *EncodeAudio( encoder_t *p_enc, aout_buffer_t *p_aout_buf )
+{
+    encoder_sys_t *p_sys = p_enc->p_sys;
+    int16_t *p_buffer = (int16_t *)p_aout_buf->p_buffer;
+    int i_samples = p_aout_buf->i_nb_samples;
+    block_t *p_chain = NULL;
+    AACENC_ERROR erraac;
+    int i_loop_count = 0;;
+    int i_samples_left = i_samples;
+    mtime_t i_pts_out;
+
+    if ( i_samples > 0 )
+    {
+        i_pts_out = p_aout_buf->i_pts - (mtime_t)((double)1000000 *
+               (double)p_sys->i_encoderdelay /
+               (double)p_enc->fmt_out.audio.i_rate);
+        if (p_sys->i_pts_last == 0)
+            p_sys->i_pts_last = i_pts_out - (mtime_t)((double)1000000 *
+               (double)(p_sys->i_frame_size) /
+               (double)p_enc->fmt_out.audio.i_rate);
+    }
+    else
+    {
+        i_pts_out = p_sys->i_pts_last;
+    }
+
+    while ( i_samples_left >= 0 )
+    {
+      //msg_Dbg( p_enc, "i_samples_left %i, i_samples %i", 
+      //              i_samples_left, i_samples);
+      AACENC_BufDesc in_buf = { 0 }, out_buf = { 0 };
+      AACENC_InArgs in_args = { 0 };
+      AACENC_OutArgs out_args = { 0 };
+      int in_identifier = IN_AUDIO_DATA;
+      int in_size, in_elem_size;
+      int out_identifier = OUT_BITSTREAM_DATA;
+      int out_size, out_elem_size;
+      void *in_ptr, *out_ptr;
+      /* The maximum packet size is 6144 bits aka 768 bytes per channel. */
+      uint8_t outbuf[20480];
+
+      if (i_samples == 0) {
+          // this forces the encoder to purge whatever is left in the internal buffer
+          in_args.numInSamples = -1;
+      } else {
+          in_ptr = p_buffer + (i_samples - i_samples_left)*p_enc->fmt_in.audio.i_channels;
+          in_size = 2*p_enc->fmt_in.audio.i_channels*i_samples_left;
+          in_elem_size = 2;
+          in_args.numInSamples = p_enc->fmt_in.audio.i_channels*i_samples_left;
+          in_buf.numBufs = 1;
+          in_buf.bufs = &in_ptr;
+          in_buf.bufferIdentifiers = &in_identifier;
+          in_buf.bufSizes = &in_size;
+          in_buf.bufElSizes = &in_elem_size;
+      }
+      out_ptr = outbuf;
+      out_size = sizeof(outbuf);
+      out_elem_size = 1;
+      out_buf.numBufs = 1;
+      out_buf.bufs = &out_ptr;
+      out_buf.bufferIdentifiers = &out_identifier;
+      out_buf.bufSizes = &out_size;
+      out_buf.bufElSizes = &out_elem_size;
+
+      if ((erraac = aacEncEncode(p_sys->handle, &in_buf, &out_buf, &in_args, &out_args)) != AACENC_OK) {
+        if (erraac == AACENC_ENCODE_EOF) {
+            msg_Info( p_enc, "Encoding final bytes (EOF)");
+        }
+        else
+        {
+            msg_Err( p_enc, "Encoding failed: %s", aac_get_errorstring(erraac));
+            break;
+        }
+      }
+      if ( out_args.numOutBytes > 0 )
+      {
+        block_t *p_block;
+        p_block = block_New( p_enc, out_args.numOutBytes );
+        p_block->i_buffer = out_args.numOutBytes;
+        vlc_memcpy( p_block->p_buffer, outbuf, out_args.numOutBytes );
+        if (i_samples == 0)
+        {
+            // I only have the numOutBytes so approximate based on compression factor
+            double d_samples_forward = p_sys->d_compression_ratio*(double)out_args.numOutBytes;
+            i_pts_out += (mtime_t)d_samples_forward;
+            p_block->i_length = (mtime_t)d_samples_forward;
+            // TODO: It would be more precise (a few microseconds) to use d_samples_forward = 
+            // (mtime_t)1000000 * (mtime_t)p_sys->i_frame_size/(mtime_t)p_enc->fmt_out.audio.i_rate
+            // but I am not sure if the lib always outputs a full frame when 
+            // emptying the internal buffer in the EOF scenario
+        }
+        else
+        {
+            if ( i_loop_count == 0 )
+            {
+                // There can be an implicit delay in the first loop cycle because leftover bytes
+                // in the library buffer from the prior block
+                double d_samples_delay = (double)p_sys->i_frame_size - (double)out_args.numInSamples /
+                                         (double)p_enc->fmt_in.audio.i_channels;
+                i_pts_out -= (mtime_t)((double)1000000 * d_samples_delay /
+                                       (double)p_enc->fmt_out.audio.i_rate);
+                //p_block->i_length = (mtime_t)((double)1000000 * (double)p_sys->i_frame_size / 
+                //    (double)p_enc->fmt_out.audio.i_rate);
+                p_block->i_length = i_pts_out - p_sys->i_pts_last;
+            }
+            else
+            {
+                double d_samples_forward = (double)out_args.numInSamples/(double)p_enc->fmt_in.audio.i_channels;
+                double d_length = ((double)1000000 * d_samples_forward /
+                                        (double)p_enc->fmt_out.audio.i_rate);
+                i_pts_out += (mtime_t) d_length;
+                p_block->i_length = (mtime_t) d_length;
+            }
+        }
+        p_block->i_dts = p_block->i_pts = i_pts_out;
+        block_ChainAppend( &p_chain, p_block );
+        //msg_Dbg( p_enc, "p_block->i_dts %llu, p_block->i_length %llu, p_sys->i_pts_last %llu\n\ 
+        //                  out_args.numOutBytes = %i, out_args.numInSamples = %i, i_samples %i, i_loop_count %i",
+        //                  p_block->i_dts , p_block->i_length,p_sys->i_pts_last,
+        //                  out_args.numOutBytes, out_args.numInSamples, i_samples, i_loop_count);
+        if (i_samples > 0)
+        {
+            p_sys->d_compression_ratio = (double)p_block->i_length / (double)out_args.numOutBytes;
+            i_samples_left -= out_args.numInSamples/p_enc->fmt_in.audio.i_channels;
+            p_sys->i_pts_last = i_pts_out;
+        }
+      }
+      else
+      {
+          //msg_Dbg( p_enc, "aac_encode_audio: not enough data yet");
+          break;
+      }
+      if ( i_loop_count++ > 100 )
+      {
+          msg_Err( p_enc, "Loop count greater than 100!!!, something must be wrong with the encoder library");
+          break;
+      }
+    }
+
+    return p_chain;
+
+}
+
+/*****************************************************************************
+ * CloseDecoder: decoder destruction
+ *****************************************************************************/
+static void CloseEncoder( vlc_object_t *p_this )
+{
+    encoder_t *p_enc = (encoder_t *)p_this;
+    encoder_sys_t *p_sys = p_enc->p_sys;
+
+    aacEncClose(&p_sys->handle);
+    
+    free( p_sys );
+}
diff --git a/modules/stream_out/transcode/transcode.c b/modules/stream_out/transcode/transcode.c
index f9f5e64..526dbf4 100644
--- a/modules/stream_out/transcode/transcode.c
+++ b/modules/stream_out/transcode/transcode.c
@@ -155,6 +155,7 @@ static const char *const ppsz_deinterlace_type[] =
 
 static int  Open ( vlc_object_t * );
 static void Close( vlc_object_t * );
+static void PurgeAudioEncoder(sout_stream_t *p_stream, sout_stream_id_t *id);
 
 #define SOUT_CFG_PREFIX "sout-transcode-"
 
@@ -625,8 +626,9 @@ static int Del( sout_stream_t *p_stream, sout_stream_id_t *id )
     {
         switch( id->p_decoder->fmt_in.i_cat )
         {
-        case AUDIO_ES:
-            transcode_audio_close( id );
+        case AUDIO_ES: 
+            PurgeAudioEncoder(p_stream, id);
+            transcode_audio_close( id, false );
             break;
         case VIDEO_ES:
             Send( p_stream, id, NULL );
@@ -669,6 +671,20 @@ static int Del( sout_stream_t *p_stream, sout_stream_id_t *id )
     return VLC_SUCCESS;
 }
 
+static void PurgeAudioEncoder(sout_stream_t *p_stream, sout_stream_id_t *id)
+{
+    block_t *p_buffer;
+    block_t *p_block = NULL;
+    
+    if ( id->p_encoder->b_purge_required == false )
+        return;
+    p_buffer = block_New( p_stream, 512 );
+    p_block = id->p_encoder->pf_encode_audio( id->p_encoder, p_buffer );
+    if( p_block  )
+        sout_StreamIdSend( p_stream->p_next, id->id, p_block );
+    block_Release(p_buffer);
+}
+
 static int Send( sout_stream_t *p_stream, sout_stream_id_t *id,
                  block_t *p_buffer )
 {
